package documenter.engine;

import java.io.*;
import java.util.*;
import org.xml.sax.*;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.*;

//import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protegex.export.html.*;
import edu.stanford.smi.protegex.storage.rdf.configurable.*;

/**
 * Business logic class of DocGenerator tool. 
 * Model role in DocGenerator MVC implementation.
 * @author Misio
 *
 */
public class DocGeneratorModel
{
	private ArrayList<File> inputFiles;
	private ArrayList<String> inputNames;
	private String docoutpath;
	private String srcoutpath;
	private String unionoutpath;
	private String projName;
	private DocGeneratorView view;
	
	/**
	 * Default constructor.
	 *
	 */
	public DocGeneratorModel()
	{
//		System.out.println("11");
		inputFiles = new ArrayList<File>(0);
		inputNames = new ArrayList<String>(0);
		docoutpath = "";
		srcoutpath = "";
		unionoutpath = "";
		projName = "";
		this.view = null;
//		System.out.println("22");
	}
	/**
	 * Contructs Model intance which status is presented on a particular View.
	 * @param view The view on which the model should be presented
	 * @deprecated
	 */
	public DocGeneratorModel(DocGeneratorView view)
	{
		inputFiles = new ArrayList<File>(0);
		inputNames = new ArrayList<String>(0);
		docoutpath = "";
		srcoutpath = "";
		unionoutpath = "";
		projName = "";
		this.view = null;
		if (view != null)
		{
			this.view = view;
		}
	}
	
	/**
	 * 
	 * @return The path to which the documentation should be generated
	 */
	public String getDocOutPath()
	{
		return docoutpath;
	}
	
	/**
	 * Sets the MVC View component for this Model
	 * @param view The View on which the model should be presented
	 */
	public void setView(DocGeneratorView view)
	{
		if (view != null)
		{
			this.view = view;
		}
	}
	
	/**
	 * Sets input list of RDF files for DocGenerator Model 
	 * @param value The list of RDF files for which the documentation is to be generated
	 */
	public void setInput(ArrayList<String> value)
	{
		inputNames.addAll(value);
		for(String temp : value)
		{
			File ftemp = new File(temp);
			inputFiles.add(ftemp);
		}
		view.refreshGeneratorView(inputNames);
	}
	
	/**
	 * Sets the path in which the generated documenation will be placed
	 * @param value Local filesystem path
	 */
	public void setOutpath(String value)
	{
		if (!value.endsWith("\\"))
		{
			value += "\\";
		}
		srcoutpath = value + "protage_source\\";
		docoutpath = value + "docs\\";
		unionoutpath = value + "union_out\\";
		File f = new File(srcoutpath);
		f.mkdirs();
		f = new File(docoutpath);
		f.mkdirs();
		f = new File(unionoutpath);
		f.mkdirs();
	}
	
	/**
	 * Name of the documentation project. This name will be used as a root directory for the 
	 * generated documentation files and placed in the directory given in the OutPath
	 * @param value 
	 */
	public void setProjName(String value)
	{
		projName = value;
	}	
	
	/**
	 * Generates documentation
	 * @throws Exception
	 */
	public void generateCompleteModel() throws Exception
	{
		createUnionModel();
	}
	
	/**
	 * Parses class name. Gets simple class name either from full class name or from the prefixed name
	 * Simple class name is retrieved from the full class name, where full class name follows either of the two patterns:
	 * NamespaceName#SimpleClassName
	 * or
	 * &PrefixName;SimpleClassName
	 *   
	 * @param cls Class name to be parsed
	 * @param ns Namespace name
	 * @param pref Prefix name
	 * @return Simple class name
	 */
	private String cleanClassName(String cls, String ns, String pref)
	{
		String result = null;
		String clsLC = cls.toLowerCase();
		String nsLC = ns.toLowerCase();
		//TODO to be removed soon
		if(clsLC.startsWith("file:///"))
		{
			clsLC = clsLC.substring(8);
		}
		if(clsLC.startsWith("file://"))
		{
			clsLC = clsLC.substring(7);
		}
		if(clsLC.startsWith("file:/"))
		{
			clsLC = clsLC.substring(6);
		}
		if(nsLC.startsWith("file:///"))
		{
			nsLC = nsLC.substring(8);
		}
		if(nsLC.startsWith("file://"))
		{
			nsLC = nsLC.substring(7);
		}
		if(nsLC.startsWith("file:/"))
		{
			nsLC = nsLC.substring(6);
		}
		//end of to be removed soon section
		//eliminates namespace from the full class name
		if (clsLC.startsWith(nsLC + "#"))
		{
			//eliminuj namespace z nazwy klasy
			result = cls.substring(cls.lastIndexOf("#")+1);
			return result;
		}
		//eliminates prefix from the full class name
		if (pref != null)
		{
			String prefLC = pref.toLowerCase();
			if (clsLC.startsWith("&"+ prefLC +";"))
			{
				//eliminuj prefix z nazwy klasy
				result = cls.substring(cls.lastIndexOf(";"));
				return result;
			}
		}
		else
		{
			if (cls.indexOf("#") > 0)
			{
				result = cls.substring(cls.lastIndexOf("#")+1);
				return result;
			}
		}
		
		return result;
	}
	
	/**
	 * Writes model as pair of files: schema (rdfs) and instances (rdf). This pair can be used to create protege projects.
	 * @param outputModel The model returned as pair of files
	 * @param classesfilename Name of the schema file
	 * @param instancesfilename Name of the instances file
	 * @throws Exception
	 */
	private void writeForProtege(Model outputModel, String classesfilename, String instancesfilename) throws Exception
	{
		File classesf = new File(classesfilename);
		File instancesf = new File(instancesfilename);
		Model emptymodel = ModelFactory.createModel(OntModelSpec.RDFS_MEM);
		
		//rdfs output
		if (classesf.exists())
		{
			classesf.delete();
			classesf.createNewFile();
		}		
		writeModel2File(outputModel, classesf);
		
		//write the first line
		BufferedWriter instancesout = new BufferedWriter(new FileWriter(instancesf));
        instancesout.write("<?xml version='1.0'?>\n");
        emptymodel.write(instancesout, "RDF/XML-ABBREV");
        instancesout.close();
	}
	
	private void writeModel2File(Model model, File file) throws Exception
	{
		FileOutputStream fos = new FileOutputStream(file);
		RDFWriter fasterWriter = model.getWriter("RDF/XML");
//		System.out.println("I have the writer");
		fasterWriter.setProperty("showXMLDeclaration","true");
		fasterWriter.setProperty("allowBadURIs","true");
		fasterWriter.setProperty("relativeURIs","");
		fasterWriter.setProperty("tab","0");
		fasterWriter.write(model, fos, "");	
		fos.close();
	}
	
	/**
	 * Creates o model object from schema file
	 * @param file Schema file to be read
	 * @return A Model object
	 * @throws Exception
	 */
	private Model readModelFromFile(File file) throws Exception
	{
		Model model = ModelFactory.createModel(OntModelSpec.RDFS_MEM);
	    RDFReader arp = model.getReader();
        InputStream in = new FileInputStream(file);
        arp.read(model, in, "");
        in.close();
        return model;
	}
	
	/**
	 * Generates main index of the documentation
	 * @param nsFile Name of the html containing all namespace names (displayed in the left top frame of the index)
	 * @param allResFile Name of the html file containing all resources from all namespaces (displayed in the bottom left frame of the index)
	 * @param project Project name
	 */
	private void generateIndex(String nsFile, String allResFile, String project)
	{
		String filename = docoutpath + "index.html";
		try 
		{
	        BufferedWriter out = new BufferedWriter(new FileWriter(filename));
	        out.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
	        out.write("<HTML><HEAD>");
	        out.write("<meta name=\"collection\" content=\"exclude\">");
	        out.write("<TITLE>Project "+ project +" documentation</TITLE>");
	        out.write("</HEAD>");
	        out.write("<FRAMESET cols=\"30%,70%\" title=\"\" onLoad=\"top.loadFrames()\">");
	        out.write("<FRAMESET rows=\"30%,70%\" title=\"\" onLoad=\"top.loadFrames()\">");
	        out.write("<FRAME src=\"./"+ nsFile +"\" name=\"packageListFrame\" title=\"All Namespaces\">");
	        out.write("<FRAME src=\"./"+ allResFile +"\" name=\"packageFrame\" title=\"All resources\">");
	        out.write("</FRAMESET>");
	        out.write("<FRAME src=\"./pdocs/index.html\" name=\"classFrame\" title=\"Package, class and interface descriptions\" scrolling=\"yes\">");
	        out.write("<NOFRAMES>");
	        out.write("<H2>");
	        out.write("Frame Alert</H2>");
	        out.write("<P>");
	        out.write("This document is designed to be viewed using the frames feature. If you see this message, you are using a non-frame-capable web client.<BR>");
	        out.write("Link to<A HREF=\"./pdocs/index.html\">Non-frame version.</A>");
	        out.write("</NOFRAMES></FRAMESET></HTML>");
	        out.close();
	    } 
		catch (IOException e) 
		{
			e.printStackTrace();
	    }
	}
	
	/**
	 * Creates html files with all resources for each namespace in the particular documentation project
	 * @param contentList Namespaces and their contents
	 */
	private void generateNSDocs(ArrayList<NSContent> contentList)
	{
		NSContent allResources = new NSContent();
		allResources.setNamespace(projName);
		
		String filename = "";
		
		for (NSContent c : contentList)
		{
			c.createHTML(docoutpath);			
		}
		
		filename = docoutpath + projName + "_namespaces.html";
		try 
		{
	        BufferedWriter out = new BufferedWriter(new FileWriter(filename));
	        out.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
	        out.write("<HTML><HEAD>");
	        out.write("<meta name=\"collection\" content=\"exclude\">");
	        out.write("<TITLE>Project "+ projName +" namespaces</TITLE>");
	        out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"./pdocs/htmlexport.css\"/>");
	        out.write("</HEAD>");
	        //out.write("<BODY BGCOLOR=\"white\"><FONT size=\"+1\" CLASS=\"pageTitle\"> <A HREF=\"./all_"+ projName +"_resources.html\" target=\"packageFrame\"><B>Project "+ projName+ " </A></FONT>");
	        out.write("<BODY BGCOLOR=\"white\"><span CLASS=\"pageTitle\"> <A HREF=\"./all_"+ projName +"_resources.html\" target=\"packageFrame\"><B>Project "+ projName+ " </A></span>");
	        out.write("<TABLE BORDER=\"0\" WIDTH=\"100%\" SUMMARY=\"\"><TR><TD NOWRAP>");
	        
	        //content
	        for(NSContent nsc : contentList)
	        {
	        	allResources.addClasses(nsc.getClasses());
	        	allResources.addProperties(nsc.getProperties());
	        	String linkname = nsc.getPrefix() + "_resources.html";
	        	out.write("<span class=\"generalContent\"><A HREF=\"./"+ linkname +"\" title=\""+ nsc.getNamespace() +"\" target=\"packageFrame\">"+ nsc.getNamespace() +"</A></span><BR>");
	        }
	        out.write("</TD></TR></TABLE></BODY></HTML>");
	        out.close();
	        allResources.createHTML(docoutpath + "all_");
	        generateIndex(projName+"_namespaces.html", "all_"+projName+"_resources.html", projName);
	    } 
		catch (Exception e) 
		{
			e.printStackTrace();
	    }
	}
	
	/**
	 * Parses schema files and creates NSContent object for a namespace
	 * TODO check the behaviour when one namespace will be represented in several files 
	 * @param file Schema file path
	 * @return NSContent object
	 * @throws Exception
	 */
	private NSContent createNSContent(File file) throws Exception
	{
		NSContent temp = new NSContent();
		temp.setNamespace(file.toURL().toString());
		FileInputStream in = new FileInputStream(file);
		if (in == null) 
		{
            throw new IllegalArgumentException(file + " not found");
        }

		OntModel omodel = ModelFactory.createOntologyModel(OntModelSpec.RDFS_MEM);
		omodel.read(new InputStreamReader(in), "");
		ExtendedIterator ei = omodel.listClasses();
		while(ei.hasNext())
		{
			String ns = temp.getNamespace();
			String name = ei.next().toString();
			String prefix = omodel.getNsURIPrefix(ns + "#");
			String oldname = name;
			name = cleanClassName(name, ns, prefix);
			if (name == null)
			{
				throw new Exception("Strange class name format: "+ oldname);
			}
			else
			{
				temp.addClass(name);
			}
		}
		ei = omodel.listOntProperties();
		while(ei.hasNext())
		{
			String ns = temp.getNamespace();
			String name = ei.next().toString();
			String prefix = omodel.getNsURIPrefix(ns + "#");
			String oldname = name;
			name = cleanClassName(name, ns, prefix);
			if (name == null)
			{
				throw new Exception("Strange property name format: "+ oldname);
			}
			else
			{
				temp.addProperty(name);
			}
		}
		omodel.close();
		in.close();
		return temp;
	}
	
	/**
	 * Creates documentation project and initiates documentation generation
	 * @throws Exception
	 */
	private void createUnionModel() throws Exception
	{
		ArrayList<NSContent> contentList = new ArrayList<NSContent>(0);
		unionoutpath += projName;
		int counter = 0;
		for(File path : inputFiles)
		{			
			contentList.add(createNSContent(path));
			Model model = readModelFromFile(path);
			File fcurrent = new File(unionoutpath + "_" + counter + ".rdf");
			if (counter == 0)
			{				
				writeModel2File(model, fcurrent);
			}
			else
			{
				File fprevious = new File(unionoutpath + "_" + (counter - 1) + ".rdf");
				Model unionModel = readModelFromFile(fprevious);
				unionModel = ModelFactory.createUnion(unionModel, model);
				writeModel2File(unionModel, fcurrent);
				unionModel.close();
			}
			counter++;
			model.close();
		}
		
		//rdfs and rdf output
		File lastFile = new File(unionoutpath + "_" + (counter-1) + ".rdf");
		Model outputModel = readModelFromFile(lastFile);
		String classesfilename = srcoutpath + projName + ".rdfs";
		String instancesfilename = srcoutpath + projName + ".rdf";
		writeForProtege(outputModel, classesfilename, instancesfilename);
		generateFromProtege();
		generateNSDocs(contentList);
		outputModel.close();
	}
	
	/**
	 * Generates documentation for each class, property and instance using Protege API
	 * @throws Exception
	 */
	@SuppressWarnings({"unchecked","static-access"})
	private void generateFromProtege() throws Exception	
	{
		//creating project
		Collection errors = new ArrayList();
        RDFCBackend factory = new RDFCBackend();
      
        //KnowledgeBase rkb = factory.createKnowledgeBase(errors);
        Project project = Project.createNewProject(factory, errors);
        
        ArrayList<Cls> baseclasses = new ArrayList<Cls>(0);
        ArrayList<Slot> baseslots = new ArrayList<Slot>(0);
        baseclasses.addAll(project.getKnowledgeBase().getClses());
        baseslots.addAll(project.getKnowledgeBase().getSlots());
        System.out.println("Bazowych klas jest: " + baseclasses.size());
        System.out.println("Bazowych slots jest: " + baseslots.size());
        
        String baseName = projName;
		// arbitrary string
        File f = new File(srcoutpath + projName);
        String myNamespace = f.toURL().toString();
        
        @SuppressWarnings("unused") InputSource is1 = new InputSource(baseName+".rdfs");        
        @SuppressWarnings("unused") InputSource is2 = new InputSource(baseName+".rdf");
        project.setProjectFilePath(srcoutpath + baseName + ".pprj");
        factory.setSourceFiles(project.getSources(), baseName+".rdfs", 
        		baseName+".rdf", myNamespace);
        
        factory.loadKnowledgeBase(project.getKnowledgeBase(), project.getSources(), errors);
        
        //creating documentation
        ExportConfiguration econf = new ExportConfiguration();
    	econf.setProject(project);
    	ArrayList<Cls> al = new ArrayList<Cls>(0);
    	//get all classes
    	al.addAll(project.getKnowledgeBase().getClses());
    	//remove system classes
    	al.removeAll(baseclasses);
    	//remove classes which are subclasses
    	ArrayList<Cls> knownSubCls = new ArrayList<Cls>(0);
    	for(Cls c : al)
    	{
    		knownSubCls.addAll(c.getDirectSubclasses());
    	}
    	al.removeAll(knownSubCls);
    	for(Cls c : knownSubCls)
    	{
    		System.out.println("Usunalem: "+c.getBrowserText());
    	}
    	Collections.sort(al);
    	econf.setRootClasses(al);
    	
    	ArrayList<Slot> slots = new ArrayList<Slot>(0);
    	slots.addAll(project.getKnowledgeBase().getSlots()); 
    	slots.removeAll(baseslots);
    	ArrayList slotnames = new ArrayList();
    	for (Slot slot : slots)
    	{
    		slotnames.add(slot.getBrowserText());
    	}
    	slotnames.add(edu.stanford.smi.protege.model.Model.Slot.DIRECT_DOMAIN);
    	slotnames.add(edu.stanford.smi.protege.model.Model.Slot.DIRECT_SUBCLASSES);
    	slotnames.add(edu.stanford.smi.protege.model.Model.Slot.DIRECT_SUPERCLASSES);
    	slotnames.add(edu.stanford.smi.protege.model.Model.Slot.DOCUMENTATION);
    	slotnames.add(edu.stanford.smi.protege.model.Model.Slot.VALUE_TYPE);
    	econf.setSlotsToDisplay(slotnames);
    	System.out.println("Slots: " + slots.size());
    	
     	ArrayList facets = new ArrayList(0);
     	String valuefacet = "Value Type";
    	facets.add(valuefacet);
    	econf.setFacetsToDisplay(facets);
    	
    	econf.setShowInstances(true);
    	econf.setSortSubclasses(true);    	
    	econf.setOutputDir(docoutpath + "pdocs\\");
    	econf.setApplicationDir("html_export");
    	econf.setCSSPath("html_export"+ File.separator + "htmlexport.css");
    	econf.setFooterPath("html_export"+File.separator +"footer.html");
    	econf.setHeaderPath("html_export"+File.separator +"header.html");
    	
    	HTMLExport export = new HTMLExport(econf);
    	export.export();
    	System.out.println("Docs created!");
	}
}
