package documenter.engine;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;

/**
 * Represents contents of schema file. The contents of the schema files are resources 
 * which are declared in a namespace. This class represents resources declared in a particular namespace.
 * @author Misio
 *
 */
public class NSContent
{
	private String namespace;
	private String prefix;
	private ArrayList<ResourceRecord> classes;
	private ArrayList<ResourceRecord> properties;

	/**
	 * Create empty NSContent object
	 *
	 */
	public NSContent()
	{
		namespace = "";
		prefix = "";
		classes = new ArrayList<ResourceRecord>(0);
		properties = new ArrayList<ResourceRecord>(0);
	}
	
	/**
	 * 
	 * @param value Objects Namespace
	 */
	public void setNamespace(String value)
	{
		if (value.endsWith(".rdf"))
		{
			value = value.substring(0, value.lastIndexOf(".rdf"));
		}
		namespace = value;
		setPrefix();
	}
	
	/**
	 * 
	 * @return Objects Namespace
	 */
	public String getNamespace()
	{
		return namespace;
	}
	
	/**
	 * 
	 * @param value Class name
	 */
	public void addClass(String value)
	{
		ResourceRecord temp = new ResourceRecord();
		temp.setLink(prefix);
		temp.setResource(value);
		classes.add(temp);
	}
	
	/**
	 * 
	 * @param value Collection of ResourceRecord objects which represent classes 
	 */
	public void setClasses(Collection<ResourceRecord> value)
	{
		classes.clear();
		classes.addAll(value);
	}
	
	/**
	 * 
	 * @param value List of ResourceRecord objects which represent classes
	 */
	public void addClasses(ArrayList<ResourceRecord> value)
	{
		classes.addAll(value);
	}
	
	/**
	 * 
	 * @return List of ResourceRecord objects which represent classes
	 */
	public ArrayList<ResourceRecord> getClasses()
	{
		return classes;
	}
	
	/**
	 * 
	 * @param value Name of the property
	 */
	public void addProperty(String value)
	{
		ResourceRecord temp = new ResourceRecord();
		temp.setLink(prefix);
		temp.setResource(value);
		properties.add(temp);
	}
	
	/**
	 * 
	 * @param value Collection of ResourceRecord objects which represent properties
	 */
	public void setProperties(Collection<ResourceRecord> value)
	{
		properties.clear();
		properties.addAll(value);
	}
	
	/**
	 * 
	 * @param value List of ResourceRecord objects which represent properties
	 */
	public void addProperties(ArrayList<ResourceRecord> value)
	{
		properties.addAll(value);
	}
	
	/**
	 * 
	 * @return List of ResourceRecord objects which represent properties
	 */
	public ArrayList<ResourceRecord> getProperties()
	{
		return properties;
	}
	
	/**
	 * Sets the prefix for the namespace
	 *
	 */
	private void setPrefix()
	{
		prefix = namespace;
		if(prefix.indexOf("/") >= 0)
		{
			prefix = prefix.substring(prefix.lastIndexOf("/") + 1);
		}
	}
	
	/**
	 * 
	 * @return Gets the prefix for the namespace
	 */
	public String getPrefix()
	{		
		return prefix;
	}
	
	/**
	 * Prints to sysout the namespace and its classes
	 *
	 */
	public void print()
	{
		System.out.println("Namespace: " + namespace);
		Iterator i = classes.iterator();
		while (i.hasNext())
		{
			System.out.println("    Class: "+ i.next());
		}
	}
	
	/**
	 * Creates html page which displays all the resources of the namespace
	 * @param filename
	 */
	@SuppressWarnings("unchecked")
	public void createHTML(String filename)
	{
		filename += (getPrefix() + "_resources.html");
		try 
		{
	        BufferedWriter out = new BufferedWriter(new FileWriter(filename));
	        out.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
	        out.write("<HTML><HEAD>");
	        out.write("<meta name=\"collection\" content=\"exclude\">");
	        out.write("<TITLE>Namespace "+ namespace +" resources</TITLE>");
	        out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"./pdocs/htmlexport.css\"/>");
	        out.write("</HEAD>");
	        out.write("<BODY BGCOLOR=\"white\"><span CLASS=\"pageTitle\"><B>"+ namespace +"</B></span>");
	        //out.write("<TABLE BORDER=\"0\" WIDTH=\"100%\" SUMMARY=\"\"><TR><TD NOWRAP><FONT CLASS=\"FrameItemFont\">");
	        out.write("<TABLE BORDER=\"0\" WIDTH=\"100%\" SUMMARY=\"\"><TR><TD NOWRAP>");
	        
	        //content
	        Collections.sort(classes);
	        out.write("<span class=\"generalContentBold\">Classes:</span><BR>");
	        for(ResourceRecord rr : classes)
	        {
	        	String linkname = "pdocs\\"+rr.getLink() + "_" + rr.getResource()+".html";
	        	out.write("<span class=\"generalContent\"><A HREF=\"./"+ linkname +"\" title=\""+ rr.getResource() +"\" target=\"classFrame\">"+ rr.getResource() +"</A></span><BR>");
	        }
	        
	        Collections.sort(properties);
	        out.write("<br><span class=\"generalContentBold\">Properties:</span><BR>");
	        for(ResourceRecord rr : properties)
	        {
	        	String linkname = "pdocs\\slot_"+ rr.getLink() + "_" + rr.getResource()+".html";
	        	out.write("<span class=\"generalContent\"><A HREF=\"./"+ linkname +"\" title=\""+ rr.getResource() +"\" target=\"classFrame\">"+ rr.getResource() +"</A></span><BR>");
	        }
	        out.write("</TD></TR></TABLE></BODY></HTML>");
	        out.close();
	    } 
		catch (IOException e) 
		{
			e.printStackTrace();
	    }
	}
}
